<?php
/*
Plugin Name: WP-TennisWeblog
Description: Ermöglicht das Anzeigen der News von Tennis Weblog in einem Widget. Das Aussehen kann völlig individuell angepasst werden.
Author: Robert Hartl
Version: 1.0
Plugin URI: http://www.tennis-weblog.de/widget/
Author URI: http://www.hartl.info/
License: GPL
*/

/*
This widget is based on the KB Advanced RSS Widget v2.1.2 (http://adambrown.info/b/widgets/category/kb-advanced-rss/) 
by Adam R. Brown (http://adambrown.info/)
German translation inspired by Sportgate.de
*/


define('TWRSS_HOWMANY', 20);
define('TWRSS_MAXITEMS', 20);
define('TWRSS_FORCECACHE', false);
define('TWRSS_WPMU', false);


function widget_twrss_init() {


	if ( !function_exists('register_sidebar_widget') )
		return;
	if ( class_exists('sg_rss') )
		return;


	if ( !function_exists('htmlspecialchars_decode') ){
	    function htmlspecialchars_decode($text){
	        return strtr($text, array_flip(get_html_translation_table(HTML_SPECIALCHARS)));
	    }
	}
	

	if (!function_exists('attribute_escape')){
		function attribute_escape($text) {
			$safe_text = wp_specialchars($text, true);
			return apply_filters('attribute_escape', $safe_text, $text);
		}
	}


	class sg_rss{
	

		var $title;
		var $linktitle;
		var $num_items;
		var $url;
		var $output_begin;
		var $output_format;
		var $output_end;
		var $utf;
		var $icon;
		var $display_empty;
		var $reverse_order;
		
		var $number;
		var $md5;
		var $md5_option;
		var $md5_option_ts;
		
		var $link;
		var $desc;
		
		var $tokens;
		var $items;

		function load_magpie(){
			if (function_exists('fetch_rss'))
				return true;

			if (file_exists(ABSPATH . WPINC . '/rss.php') )
				require_once(ABSPATH . WPINC . '/rss.php');
			elseif (file_exists(ABSPATH . WPINC . '/rss-functions.php') )
				require_once(ABSPATH . WPINC . '/rss-functions.php');

			if (function_exists('fetch_rss'))
				return true;
			return false;
		}
		
		function load_options( $args = false ){
			if (is_array($args)){
				$this->number = 0;
				$options[0]['url'] = $args[0];
				$options[0]['items'] = $args[1];
				$options[0]['output_format'] = $args[2];
				$options[0]['utf'] = $args[3];
			}else{
				$options = get_option('widget_twrss');
			}
			
			$this->title = $options[$this->number]['title'];
			$this->linktitle = $options[$this->number]['linktitle'];

			$this->num_items = (int) $options[$this->number]['items'];
			if ( empty($this->num_items) || ($this->num_items < 1) || ($this->num_items > TWRSS_MAXITEMS) )
				$this->num_items = TWRSS_MAXITEMS;
			
			$this->url = $options[$this->number]['url'];
			if ( empty($this->url) || false===strpos($this->url,'http') )
				return false;

			while ( strstr($this->url, 'http') != $this->url )
				$this->url = substr($this->url, 1);

			$this->md5 = md5($this->url);
			$this->md5_option = 'rss_' . $this->md5;
			$this->md5_option_ts = $this->md5_option . '_ts';
			
			$this->output_begin = $options[$this->number]['output_begin'];
			$this->output_format = $options[$this->number]['output_format'];
			$this->output_end = $options[$this->number]['output_end'];
			$this->utf = $options[$this->number]['utf'];
			$this->display_empty = (1==$options[$this->number]['display_empty']) ? true : false;
			$this->reverse_order = (1==$options[$this->number]['reverse_order']) ? true : false;
			
			$this->icon = $options[$this->number]['icon'];
			
			if ( empty($this->output_format) )
				$this->output_format = '<li><a class="twrsswidget" target="_blank" href="^link$" title="^title$">^title$</a></li>';

			return true;
		}
		
		function force_cache(){

			global $userdata;
			if ( ('flush' == $_GET['twrss_cache']) && ($userdata->user_level >= 7) ){
				delete_option( $this->md5_option );
				return;
			}

			if ( ! TWRSS_FORCECACHE )
				return;
			$cachetime = get_option( $this->md5_option_ts );
			if ( $cachetime < ( time() - 3600 ) )
				delete_option( $this->md5_option );
		}
		
		function get_feed(){
			$rss = @fetch_rss($this->url);
			
			/*
			$this->link = clean_url(strip_tags($rss->channel['link']));
			while( strstr($this->link, 'http') != $this->link )
				$this->link = substr( $this->link, 1 );
			*/

			$this->desc = attribute_escape(strip_tags(html_entity_decode($rss->channel['description'], ENT_QUOTES)));

			$this->url = clean_url(strip_tags($this->url));
			
			if ( ('link'==$this->linktitle) && $this->title )
				$this->title = '<a href="http://www.tennis-weblog.de/">'. $this->title .'</a>';
							
			if ('' != $this->icon)
				$this->title = $this->title;
			

			if ( is_array($rss->items) && !empty( $rss->items ) ){
				if ($this->reverse_order)
					$rss->items = array_reverse( $rss->items );
				$rss->items = array_slice($rss->items, 0, $this->num_items);

				$this->items = '';

				foreach( $rss->items as $item ){
					$find = array();
					foreach( $this->tokens as $token ){
						$replace = '';
						if ( is_array($item[ $token['field'] ]) ){
							if ( $token['opts']['subfield'] ){
								$replace = $item[ $token['field'] ][ $token['opts']['subfield'] ];
								$replace = $this->item_cleanup( $replace, $token['opts'] );
							}elseif ( $token['opts']['loop'] ) {
								foreach( $item[ $token['field'] ] as $subfield ){
									$subfield = $this->item_cleanup( $subfield, $token['opts'] );
									$replace .= $token['opts']['beforeloop'] . $subfield . $token['opts']['afterloop'];
									}
							}
						}else{
							$replace = $item[ $token['field'] ];
							$is_url = ('link'==$token['slug']) ? true : false;
							$replace = $this->item_cleanup( $replace, $token['opts'], $is_url );
						}
						$find[ $token['slug'] ] = $replace;
					}
					$keys = array_keys( $find );
					$vals = array_values( $find );
					$this->items .= str_replace( $keys, $vals, $this->output_format );
				}
				
				if ($this->utf)
					$this->items = utf8_encode( $this->items );

			}else{
				if ($this->display_empty){
					if ( '<li' === substr( $this->output_format, 0, 3 ) )
						$this->items = '<li>' . __( 'Ein Fehler ist aufgetreten! Der Feed ist möglicherweise nicht erreichbar. Versuchen Sie es später noch einmal.' ) . '</li>';
					else
						$this->items = __( 'Ein Fehler ist aufgetreten! Der Feed ist möglicherweise nicht erreichbar. Versuchen Sie es später noch einmal.' );
				}else{
					$this->items = '';
					return false;
				}
			}
			return true;
		}

		function item_cleanup($text,$opts=false,$url=false){
			if (TWRSS_WPMU || !is_array($opts) || !array_key_exists('bypasssecurity',$opts)){
				if ($url)
					$text = clean_url(strip_tags($text));
				else
					$text = str_replace(array("\n", "\r"), ' ', attribute_escape(strip_tags(html_entity_decode($text, ENT_QUOTES))));
			}

			if (!is_array($opts))
				return $text;
			extract($opts, EXTR_SKIP);
			
			if ($date){
				$text = $this->make_date($text,$date);
			}
			
			$ltrim = (is_numeric($ltrim) && 0<$ltrim) ? (int) $ltrim : null;
			if (is_int($ltrim))
				$text = substr( $text, $ltrim );

			$trim = (is_numeric($trim) && 0<$trim) ? (int) $trim : null;
			if (is_int($trim))
				$text = substr( $text, 0, $trim );

			return $text;
		}

		function make_date($string, $format){
			$time = strtotime( $string );
			if (false===$time || -1===$time)
				return $string;
			return date( $format, $time );
		}
		
		function detect_tokens(){
			if (''==$this->output_format)
				return false;
			preg_match_all( '~\^([^$]+)\$~', $this->output_format, $matches, PREG_SET_ORDER);
			if (!is_array($matches) || empty($matches))
				return false;
			$tokens = array();
			$used = array();
			foreach( $matches as $match ){
				if ( in_array($match[0], $used) )
					continue;
				$used[] = $match[0];

				$token = array();
				$token['slug'] = $match[0];
				$token['opts'] = array();

				if ( strpos($match[1], '[opts:') ){
					$explode = explode( '[opts:', $match[1], 2 );
					$match[1] = $explode[0];
					$opts = substr( $explode[1], 0, -1 );
					parse_str( $opts, $options );
					$token['opts'] = array_merge( $token['opts'], $options );
				}

				if ( strpos($match[1], '%%') ){
					$explode = explode( '%%', $match[1] );
					$match[1] = $explode[0];
					$token['opts']['trim'] = $explode[1];
				}

				if ( strpos($match[1], '=>') ){
					$explode = explode( '=>', $match[1], 2);
					$match[1] = $explode[0];
					$token['opts']['subfield'] = $explode[1];
				}elseif( strpos($match[1], '||') ){
					$explode = explode( '||', $match[1], 3);
					$match[1] = $explode[0];
					$token['opts']['loop'] = true;
					$token['opts']['beforeloop'] = $explode[1];
					$token['opts']['afterloop'] = $explode[2];
				}

				$token['field'] = $match[1];

				$tokens[] = $token;
			}

			if (empty($tokens))
				return false;

			$this->tokens = $tokens;
			return true;
		}

		function prepare_widget(){
			if (!$this->load_magpie())
				return false;
			if (!$this->load_options())
				return false;
			if (!$this->detect_tokens())
				return false;
			$this->force_cache();
			if (!$this->get_feed())
				return false;
			return true;
		}
		
		function display_widget( $args, $num = 1 ){
			$this->number = $num;
			if (!$this->prepare_widget()){
				echo '<!-- Das WP-TennisWeblog RSS Widget konnte nicht vorbereitet werden. -->';
				return;
			}
			extract( $args );
			echo $before_widget;
			if ( $this->title )
				echo $before_title . $this->title . $after_title;
			echo $this->output_begin;
			echo $this->items;
			echo $this->output_end;
			echo $after_widget;
		}
		
		function display_template( $url, $format, $numItems=10, $utf=false, $echo=true ){
			if (!$this->load_magpie())
				return false;
			if (!$this->load_options( array($url, $numItems, $format, $utf) ))
				return false;
			if (!$this->detect_tokens())
				return false;
			$this->force_cache();
			if (!$this->get_feed())
				return false;
			if ($echo)
				echo $this->items;
			else
				return $this->items;
		}
	}

	function widget_twrss( $args, $number = 1 ){
		global $sg_rss;
		$sg_rss->display_widget( $args, $number );
	}

	function sg_rss_template($url, $format, $numItems=10, $utf=false, $echo=true){
		global $sg_rss;
		return $sg_rss->display_template( $url, $format, $numItems, $utf, $echo );
	}

	function widget_twrss_control($number) {
		$options = get_option('widget_twrss');
		$newoptions = $options;

		if ( $_POST["twrss-submit-$number"] ) {
			$newoptions[$number]['items'] = (int) $_POST["twrss-items-$number"];
			$newoptions[$number]['url'] = strip_tags(stripslashes($_POST["twrss-url-$number"]));
			if ( file_exists(dirname(__FILE__) . '/rss.png') ){
				$icon = str_replace(ABSPATH, get_settings('siteurl').'/', dirname(__FILE__)) . '/rss.png';
			}else{
				$icon = get_settings('siteurl').'/wp-includes/images/rss.png';
			}
			$newoptions[$number]['icon'] = strip_tags(stripslashes($icon));
			if (TWRSS_WPMU){
				$newoptions[$number]['title'] = trim(strip_tags(stripslashes($_POST["twrss-title-$number"])));
			}else{
				$newoptions[$number]['title'] = trim( stripslashes($_POST["twrss-title-$number"]) );
			}
			$newoptions[$number]['linktitle'] = "link";
			$newoptions[$number]['display_empty'] = (1==$_POST["twrss-hideempty-$number"]) ? 0 : 1;
			$newoptions[$number]['reverse_order'] = (1==$_POST["twrss-reverseorder-$number"]) ? 1 : 0;
			$newoptions[$number]['output_format'] = htmlspecialchars_decode( stripslashes($_POST["twrss-output_format-$number"]) );
			$newoptions[$number]['output_begin'] = htmlspecialchars_decode( stripslashes("<ul id=\"twidget-list\">") );
			$newoptions[$number]['output_end'] = htmlspecialchars_decode( stripslashes("</ul><p class=\"twidget-footer\"><small><a href=\"http://www.tennis-weblog.de\" target=\"_blank\" title=\"mehr Tennis Nachrichten\">&raquo; mehr Tennis News</a></small></p>") );
			$newoptions[$number]['utf'] = null;


		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_twrss', $options);
		}
		$url = htmlspecialchars($options[$number]['url'], ENT_QUOTES);
		$items = (int) $options[$number]['items'];
		$title = htmlspecialchars($options[$number]['title'], ENT_QUOTES);
		$display_empty = (int) $options[$number]['display_empty'];
		$reverse_order = (int) $options[$number]['reverse_order'];
		$output_format = htmlspecialchars($options[$number]['output_format'], ENT_QUOTES);

		$output_format1 = htmlspecialchars("<li><a class='twrsswidget' href='^link\$' title='^title\$'>^title\$</a></li>", ENT_QUOTES);
		$output_format2 = htmlspecialchars("<li><a class='twrsswidget' href='^link\$' title=''>^title\$</a></li>", ENT_QUOTES);

		if ( empty($items) || $items < 1 ){
			$items = 10;
		}
		if ( '' == $title ){
			$title = "Tennis News";
		}
		if ( '' == $output_format ){
			$output_format = $output_format1;
		}
		if ( '' == $url ){
			$url = "http://www.tennis-weblog.de/feed/";
		}
		
			?>
				<div id="sg_rss_settings_<?php echo $number; ?>">
					<input style="width: 400px;" id="twrss-url-<?php echo $number; ?>" name="twrss-url-<?php echo $number; ?>" type="hidden" value="<?php echo $url; ?>" />
					<table>
					<tr>
						<td><?php _e('Titel (optional):', 'sgwidgets'); ?> </td>
						<td colspan="3"><input style="width: 400px;" id="twrss-title-<?php echo "$number"; ?>" name="twrss-title-<?php echo "$number"; ?>" type="text" value="<?php echo $title; ?>" /></td>
					</tr>
					<!--<tr>
            <td><label for="twrss-output_format-<?php echo "$number"; ?>"><?php _e('Art der Anzeige:', 'sgwidgets'); ?></label></td>
  					<td colspan="3"><select id="twrss-output_format-<?php echo "$number"; ?>" name="twrss-output_format-<?php echo "$number"; ?>">
  					  <option value="<?php echo $output_format1; ?>" <?php if($output_format == $output_format1) { echo 'selected="selected"'; } ?> >Nur Titel</option>
  					  <option value="<?php echo $output_format2; ?>" <?php if($output_format == $output_format2) { echo 'selected="selected"'; } ?> >Titel und Kurztext</option>
            </select></td>
					</tr>-->
					<tr>
						<td><label for="twrss-items-<?php echo $number; ?>"><?php _e('Anzahl anzuzeigender News:', 'sgwidgets'); ?> </label></td>
						<td colspan="3"><select id="twrss-items-<?php echo $number; ?>" name="twrss-items-<?php echo $number; ?>"><?php for ( $i = 1; $i <= twrss_MAXITEMS; ++$i ) echo "<option value='$i' ".($items==$i ? "selected='selected'" : '').">$i</option>"; ?></select></td>
					</tr>
					<tr>
						<td><label for="twrss-hideempty-<?php echo $number; ?>"><?php _e('Das Widget verbergen wenn der Feed nicht erreichbar ist?', 'sgwidgets'); ?> </label></td>
						<td><input type="checkbox" name="twrss-hideempty-<?php echo $number; ?>" id="twrss-hideempty-<?php echo $number; ?>" value="1" <?php if ( 1!=$display_empty ){ echo 'checked="checked"'; } ?> /> </td>
					</tr>
					<!--<tr>
						<td><label for="twrss-reverseorder-<?php echo $number; ?>"><?php _e('Reihenfolge der News im Feed umkehren?', 'sgwidgets'); ?> </label></td>
						<td><input type="checkbox" name="twrss-reverseorder-<?php echo $number; ?>" id="twrss-reverseorder-<?php echo $number; ?>" value="1" <?php if ( 1===$reverse_order ){ echo 'checked="checked"'; } ?> /> </td>
					</tr>-->
					</table>
					<input type="hidden" id="twrss-submit-<?php echo "$number"; ?>" name="twrss-submit-<?php echo "$number"; ?>" value="1" />
				</div>
	<?php
	}
	

	function widget_twrss_setup() {
		$options = $newoptions = get_option('widget_twrss');
		if ( isset($_POST['twrss-number-submit']) ) {
			$number = (int) $_POST['twrss-number'];
			if ( $number > TWRSS_HOWMANY ) $number = TWRSS_HOWMANY;
			if ( $number < 1 ) $number = 1;
			$newoptions['number'] = $number;
		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_twrss', $options);
			widget_twrss_register($options['number']);
		}
	}

	function widget_twrss_page() {
		$options = $newoptions = get_option('widget_twrss');
	?><!--
		<div class="wrap">
			<form method="POST">
				<h2>TennisWeblog Widgets</h2>
				<p style="line-height: 30px;"><?php _e('Wieviele TennisWeblog Widgets möchten Sie gleichzeitig verwenden?', 'sgwidgets'); ?>
				<select id="twrss-number" name="twrss-number" value="<?php echo $options['number']; ?>">
	<?php for ( $i = 1; $i <= TWRSS_HOWMANY; ++$i ) echo "<option value='$i' ".($options['number']==$i ? "selected='selected'" : '').">$i</option>"; ?>
				</select>
				<span class="submit"><input type="submit" name="twrss-number-submit" id="twrss-number-submit" value="<?php _e('Save'); ?>" /></span></p>
			</form>
		</div>-->
	<?php
	}

	function widget_twrss_register() {
		global $wp_version;
		$options = get_option('widget_twrss');
		$number = $options['number'];
		if ( $number < 1 ) $number = 1;
		if ( $number > TWRSS_HOWMANY ) $number = TWRSS_HOWMANY;
		for ($i = 1; $i <= TWRSS_HOWMANY; $i++) {
			$name = array('Tennis Weblog News %s', null, $i);
			if ( '2.2' == $wp_version ){
				register_sidebar_widget($name, $i <= $number ? 'widget_twrss' : /* unregister */ '', '', $i);
				register_widget_control($name, $i <= $number ? 'widget_twrss_control' : /* unregister */ '', 700, 580, $i);
			}elseif ( function_exists( 'wp_register_sidebar_widget' ) ){
				$id = "sg-rss-$i";
				$dims = array('width' => 700, 'height' => 580);
				$class = array( 'classname' => 'widget_twrss' );
				$name = sprintf(__('Tennis Weblog News'), $i);
				wp_register_sidebar_widget($id, $name, $i <= $number ? 'widget_twrss' : /* unregister */ '', $class, $i);
				wp_register_widget_control($id, $name, $i <= $number ? 'widget_twrss_control' : /* unregister */ '', $dims, $i);
			}else{
				register_sidebar_widget($name, $i <= $number ? 'widget_twrss' : /* unregister */ '', $i);
				register_widget_control($name, $i <= $number ? 'widget_twrss_control' : /* unregister */ '', 700, 580, $i);
			}
		}
	
		add_action('sidebar_admin_setup', 'widget_twrss_setup');
		add_action('sidebar_admin_page', 'widget_twrss_page');

	}

	$GLOBALS['sg_rss'] = new sg_rss();
	widget_twrss_register();

}

function widget_twrss_troubleshooter(){
	if ( !($_GET['twrss']) )
		return;

	global $userdata;
	if ( $userdata->user_level >= 7 ){
		if ( file_exists(ABSPATH . WPINC . '/rss.php') )
			require_once(ABSPATH . WPINC . '/rss.php');
		else
			require_once(ABSPATH . WPINC . '/rss-functions.php');
		$rss = @fetch_rss($_GET['twrss']);
		$out = "<html><head><title>WP-TennisWeblog RSS Problemlöser</title></head><body><div style='background:#cc0;padding:1em;'><h2>WP-TennisWeblog RSS Problemlöser</h2><p>Hierunter sollten Sie den Feed sehen wie Wordpress ihn an das WP-TennisWeblog RSS Widget übergibt.</p></div><pre>";
		$out .= htmlspecialchars( print_r($rss->items, true) );
		$out .= "</pre></body></html>";
		print $out;
		die;
	}else{
		print "<p>Sie müssen als Administrator angemeldet sein.</p>";
		die;
	}
	return;
}

add_action('widgets_init', 'widget_twrss_init');
add_action('template_redirect', 'widget_twrss_troubleshooter');

?>